import com.stencyl.utils.Utils;

class PhonemeTranscription
{

	// For espeak sepcial Kirschenbaum
	public static var xSampaToKirshenBaumMap:Map<String,String> = 
	[
		"r" => "r<trl>",  // alveolar trill
		"r\\" => "r",    // alveolar approximant ɹ
		"{"  => "a",     // near-open front unrounded vowel 	æ 
		"Q" => "O", //open back rounded vowel 	ɒ 	Q 	A.
		"O" => "O:",
		'"' => "'" ,// primary stress
		"%" => "," // secondary stress
	];


	// For pico
	public static var xSampaToPicoXSampa:Map<String,String> = 
	[
		// affricates
		"dZ" => "d_Z",  
		"tS" => "t_S",  
		// monophtongs
		"aU"  => "a_U",
		// diphtongs
		"@U"  =>"@_U",
		"I@"  =>"I_@",
		"OI"  =>"O_I",
		"U@"  =>"U@",
		"aI"  => "a_I",
		"e@"  => "e_@",
		"eI" => "e_I",
		"oU"  => "o_U"
	];

	// For Ivona
	// http://developer.ivona.com/en/ttsresources/phonesets/phoneset-en_gb.html
	public static var xSampaToIvonaXSampa:Map<String,String> = 
	[
		"@l" => "l=",  
		"@m" => "m=",
		"@n" => "n=",    

	];

	public static var xSampaToArpabet:Map<String,String> = 
	[
		// monophtongs
		"Q" => "ao1",
		"a" => "aa1",
		"A" => "aa1",
		"A:" => "aa1",
		"i:" => "iy1",
		"u:" => "uw1",
		"E" => "eh1",
		"I" => "ih1",
		"U" =>  "uh1",
		"V" => "ah1",
		"@" => "ah1",
		"{" => "ae1",

		// diphtongs
		"eI" => "ei1",
		"aI" => "ay1",
		"oU" => "ow1",
		"aU" => "aw1",
		"OI" => "oy1",

		// r-coloured vowels
		"3:" => "er1",
		"I@" => "iy1 r",

		// consonnants stops
		"p" => "p",
		"b" => "b",
		"t" => "t",
		"d" => "d",
		"k" => "k",
		"g" => "g",
		
		// consonnants affricates
		"tS" => "ch",
		"dZ" => "jh",

		// consonnants fricatives
		"f" => "f",
		"v" => "v",
		"T" => "th",
		"D" => "dh",
		"s" => "s",
		"z" => "z",
		"S" => "sh",
		"Z" => "zh",
		"h" => "hh",

		// consonnants nasals
		"m" => "m",
		"n" => "n",
		//
		"N" => "ng",

		// consonnants liquids
		"l" => "l",
		"@l" => "el",
		"r\\"  => "r",

		// semivowels
		"j" => "y",
		"w" => "w"





	];
	

	public static function xSampaToKirshenBaum(xSampa:String)
	{

		var kirshenBaum:String = "";
		for ( i in 0...xSampa.length)
		{
			var char = xSampa.charAt(i);
			var charTo = xSampaToKirshenBaumMap.get(char);

			if (charTo != null)
			{
				kirshenBaum += charTo;
			}
			else
			{
				kirshenBaum += char;
			}

		}

		return kirshenBaum;
	}

	public static function xSampaToPico(xSampa:String)
	{
		var regString = "";
		var keys = Utils.mapToList(xSampaToPicoXSampa, "keys");
		for (i in 0...keys.length)
		{
			var key = keys[i];
			if (i==0)
			{
				regString = "(";
			}
			regString += key;
			if (i==keys.length-1)
			{
				regString += ")";
			}
			else
			{
				regString +="|";
			}
		}

		var r = new EReg(regString, "g");
		
    	var s2 = r.map(xSampa, function(r) {
        var match = r.matched(0);
        return xSampaToPicoXSampa.get(match);
    });
    	return s2;
	}


	public static function xSampaToArpabetConvert(xSampa:String)
	{
		var arpabet:String = "";
		var pos:Int = 0;
		var keys = Utils.mapToList(xSampaToArpabet, "keys");
		keys.sort(function(a:String, b:String):Int {
  if (a.length < b.length) {
    return 1;
  }
  else if (a.length > b.length) {
    return -1;
  } else {
    return 0;
  }
});

		trace(keys);

		while (pos < xSampa.length)
		{
			var hasFoundXSampa:Bool = false;
			for (xs in keys)
			{
				if (xs == xSampa.substr(pos, xs.length))
				{
					trace(xs + "__" +xSampaToArpabet.get(xs));
					hasFoundXSampa = true;
					arpabet = arpabet + xSampaToArpabet.get(xs) + " ";
					// should check if vowel for stress
					pos += xs.length;
					break;
				}

			}

			trace(pos);

			if (!hasFoundXSampa)
			{
				++pos;
			}


		}
			trace(arpabet);
		return arpabet;
	}
}
	