package;

import openfl.Lib;

#if (openfl >= "4.0.0")  
import lime.system.JNI;
import Sys;
#elseif android
import openfl.utils.JNI;
import Sys;
#end
import com.stencyl.Engine;
import com.stencyl.behavior.Script;
import com.stencyl.behavior.Script.*;
import com.stencyl.behavior.TimedTask;


class TTSAndroid
{	

	#if android
	private static var funcSayText:Dynamic;
	private static var funcCheck:Dynamic;
	private static var funcSetLang:Dynamic;
	private static var funcSetPitch:Dynamic;
	private static var funcSetSpeechRate:Dynamic;
	private static var funcIsSpeaking:Dynamic;
	private static var funcGetLang:Dynamic;
	private static var funcGetActualLang:Dynamic;
	private static var funcStartEngine:Dynamic;
	private static var funcCloseEngine:Dynamic;
	private static var funcStopEngine:Dynamic;
	private static var funcGetEngines:Dynamic;
	private static var funcAddEarcon:Dynamic;
	private static var funcPlayEarcon:Dynamic;
	private static var funcSync:Dynamic; 
	private static var funcGetDefault:Dynamic; 

	public static var tTSEngineName:String = "";
	private static var tTSEngineType:String = "";

	private static var supportsSSMLTags:Bool;
	public static var supportsPhonemeTag:Bool;

	//private static 
	#end


	/*
	com.google.android.tts,com.svox.pico,edu.cmu.cs.speech.tts.flite,com.redzoc.ramees.tts.espeak
	 */


// ---------------------------------- Functions used by other functions in this file ------------------------------------------//




   // The following should be the right solution but it works inconsistently in Java
		  //    return ESCAPE_PATTERN.replace(text, "\\\\$1");
	public static function XSAMPAEscape(text : String) : String
    {
    	var XMLXSAMPA_ESCAPE_PATTERN = ~/\\/g;
		var s =  XMLXSAMPA_ESCAPE_PATTERN.map(
            text , function(ereg){ return '\\' + ereg.matched(0);});
		return s;
    }

	/**
	 * Converts a normal string to a string composed of charcodes separated by commas
	 */
	public static function convertToCharCode(oldString:String):String
	{

		var realCount = 0;
		var returnedString:String = "";
		for (i in 0...(oldString.length))
		{	
			var charCode:Int = oldString.charCodeAt(i);
			if(i < realCount)
			{
				continue;
			}
			if (charCode == 126)   // or ~
			{
				if(oldString.charAt(i + 1) == 'x')
				{
					var unicodeChar = oldString.substring(i + 2, i + 6);
					charCode = Std.parseInt("0x" + unicodeChar);
					realCount += 5;
				}
			}

			returnedString = returnedString + charCode +",";
			realCount ++;
		}
		return returnedString;  
	}



//---------------------------------------- Functions that appear in blocks ------------------------------------------------------------//

	


	/**
	 * Set Engine
	 */
	public static function setEngine(name:String)
	{
		tTSEngineName = name;
	}


	/**
	*	Close Engine
	*/
	public static function closeEngine():Void
	{
		#if android
		if (funcCloseEngine == null)
		{
			funcCloseEngine = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","closeEngine","()V",true);
		}
		funcCloseEngine();
		#end
	}

	/**
	*	Stop Engine
	*/
	public static function stopEngine():Void
	{
		#if android
		if (funcStopEngine == null)
		{
			funcStopEngine = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","closeEngine","()V",true);
		}
		funcStopEngine();
		#end
	}


	/**
	*  Flush text speech engine and make it pronounce text
	*/
	public static function sayText(textToSay:String, choice:Int, idString:String=""):Void
	{
		
		#if android
		if (funcSayText == null)
		{
			funcSayText = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","sayText","(Ljava/lang/String;Ljava/lang/String;I)V",true);
		}

		//  check supports ssml tag, if supports ssml tag add <speak> tag
		
		var text:String = supportsSSMLTags ? textToSay : "<speak>"+ textToSay +"</speak>"  ; 
		
		
		if (text!="")
		{
			var args = new Array<Dynamic>();
			args.push(text);//convertToCharCode(textToSay));
			args.push(idString);
			args.push(choice);
			funcSayText(args);
		}
		else
		{
			setGameAttribute("TTSLastSaidId", idString);
			TTS.tTSLastSaidId = idString;
			shoutToScene("_customEvent_" + "TTSHasFinishedPlaying");
		}
		#end
	}

		/**
	 *  String for XSAMPA so that it can be pronounced by TTS
	 */
	public static function adaptedXSAMPAString(s:String):String
	{
		var adaptedText:String;
		#if android
			if (!(TTSAndroid.tTSEngineName.indexOf("flite")==-1)&&(GlobalSettings.usedLanguage.iso)== "en")
			{
				adaptedText = "[[" + PhonemeTranscription.xSampaToArpabetConvert(s)+ "]]";
			}
			else if (TTSAndroid.supportsPhonemeTag)
			{

				adaptedText = "<phoneme alphabet=\"x-sampa\" ph=" + "'" + XSAMPAEscape(PhonemeTranscription.xSampaToPico(s)) + "'"+ "/>"	;	
			}
			else
			{
				adaptedText = s;
			}

		#end
		return adaptedText;
	}

	/**
	 *  Check if supports XSAMPA
	 */
	
	public static function hasXSAMPASupport():Bool
	{
		return supportsPhonemeTag;
	}
			

	public static function getDefaultEngine():String
	{
		#if android
		if (funcGetDefault == null)
		{
			funcGetDefault = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","getDefaultEngine","()Ljava/lang/String;",true);
		}
		
		return funcGetDefault();
		#end
	}

	public static function wrapIntoXSampaTag(xSampa:String)
	{
		var taggedText:String = "";
		switch (tTSEngineType) {
			case "espeak": taggedText =  "[[" + xSampa + "]]";   // should convert it before a few differences
			case "pico": taggedText =  "<speak><phoneme alphabet=\"xsampa\" ph=" + "'" + xSampa + "'"+ "/></speak>";
			case "ivona" : taggedText = "<speak><phoneme alphabet=\"x-sampa\" ph=" + "'" + xSampa + "'"+ "/></speak>";
			default: taggedText =  xSampa;
		}
		return taggedText;
	} 

	/**
	*	TODO Check if mobile has TTS engine, if not propose to install
	*/
	public static function checkHasTTS():Void
	{
		#if android
		if (funcCheck == null) 
		{
			funcCheck = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","checkHasTTS","()V",true);
		}
		funcCheck([]);
		#end
	}

	/**
	*   Set locale or language of TTS
	*/
	public static function setLanguageLocale(lang_variant:Int, lang:Int, lang_locale:String):Void
	{
		#if android
		var chosenOne:String = lang_locale ;
		if ((lang_locale != null) && (lang_locale !=""))
		{
			chosenOne = lang_locale;
		} 
		else if (lang != 0) 
		{
			switch(lang)
			{
				case 0: chosenOne = "";
				case 1: chosenOne = "en";
				case 2: chosenOne = "fr";
				case 3: chosenOne = "de";
				case 4: chosenOne = "zh";
				case 5: chosenOne = "nl";
				case 6: chosenOne = "it";
				case 7: chosenOne = "ja";
				case 8: chosenOne = "es";
				case 9: chosenOne = "ru";
				case 10: chosenOne = "ko";
				case 11: chosenOne = "pl";
				case 12: chosenOne = "default";
				default: chosenOne = "";
			}
		}
		else
		{
			switch(lang_variant)
			{
				case 0: chosenOne = "";
				case 1: chosenOne = "en_US";
				case 2: chosenOne = "en_UK";
				case 3: chosenOne = "fr_FR";
				case 4: chosenOne = "de_DE";
				case 5: chosenOne = "zh_CN";
				case 6: chosenOne = "zh_TW";
				case 7: chosenOne = "nl_NL";
				case 8: chosenOne = "it_IT";
				case 9: chosenOne = "ja_JP";
				case 10: chosenOne = "es_ES";
				case 11: chosenOne = "ru_RU";
				case 12: chosenOne = "ko_KR";
				case 13: chosenOne = "pl_PL";
				case 14: chosenOne = "default";
				default: chosenOne = "";
			}
		}
		if (funcSetLang ==null)
		{
			funcSetLang = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","setLanguageAttempt","(Lorg/haxe/lime/HaxeObject;Ljava/lang/String;Ljava/lang/String;)V",true);
		}
		var args = new Array<Dynamic>();
		args.push(new TTSAndroid());
		args.push(chosenOne);
		args.push(tTSEngineName);
		funcSetLang(args);

		if (tTSEngineName == "")
		{
			tTSEngineName = getDefaultEngine();
		}
		adaptSSMLToEngine(tTSEngineName);
		#end
	}

	/**
	*	Set Pitch of TTS, 1 is default
	*/
	public static function setPitch(pitch:Float):Void
	{
		#if android
		if (funcSetPitch)
		{
			funcSetPitch = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","changePitch","(F)V",true);
		}
		funcSetPitch([pitch]);
		#end
	}

	/*
	*	Set Speech Rate of TTS, 1 is default
	*/
	public static function setSpeechRate(speechRate:Float):Void
	{
		#if android
		if (funcSetSpeechRate == null)
		{
			funcSetSpeechRate = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","changeSpeechRate","(F)V",true);
		}
		funcSetSpeechRate([speechRate]);
		#end
	}

	/**
	*   Checks if TTS is speaking
	*/
	public static function isSpeaking():Bool
	{
		#if android
		if (funcIsSpeaking == null)
		{
			funcIsSpeaking = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","isTTSSpeaking","()Z", true);
		}
		return funcIsSpeaking();
		#else
		return false;
		#end 
	}

	/**
	*	Check all available locale languages for initialised TTS engine
	*/
	public static function getLangAvailable(choice:Int):Array<Dynamic>
	{
		#if android
		if (funcGetLang == null)
		{
			funcGetLang = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","getLanguages","()V", true);
		}
		funcGetLang([]);
		Sys.sleep(1);
		switch(choice)
		{
			case 0: return langIdsList;
			case 1: return langDisplayNames;
			default: return null;
		}
		#else
		return [];
		#end
	}

	/**
	*	Get actual language
	*/
	public static function getLang():String 
	{
		#if android
		if (funcGetActualLang == null)
		{
			funcGetActualLang = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","getLang","()Ljava/lang/String;", true);
		}
		return funcGetActualLang();
		#else
		return "";
		#end
	}



	/**
	*	Check all installed engines
	*   From Api 14
	*/
	public static function getInstalledEngines():Array<Dynamic>
	{
		#if android
		if (funcGetEngines == null)
		{
			funcGetEngines = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","getInstalledEngines","()Ljava/lang/String;");
		}
		var temp:String = funcGetEngines();
		trace("installed engines are " + temp.split(";"));
		return temp.split(";");
		#else
		return [];
		#end
	}

    /**
    *	Add Earcon, mapping between string of text and sound file
    */
    public static function addEarcon(soundName:String, soundPath:String):Void
    {
    	#if android
    	if (funcAddEarcon == null)
    	{
    		funcAddEarcon = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","addCustomEarcon","(Ljava/lang/String;Ljava/lang/String;)V",true);
    	}
    	
    	funcAddEarcon([soundName,soundPath]);
    	#end
    }

    /**
    *	Adds earcon to queue
    */
    public static function playEarcon(soundName:String):Void
    {
    	#if android
    	if (funcPlayEarcon == null)
    	{
    		funcPlayEarcon = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","playCustomEarcon","(Ljava/lang/String;)V",true);
    	}
    	funcPlayEarcon([soundName]);
    	#end
    }

    /**
    *
    */
    public static function synthetiseToFile(text:String, location:Int, folder:String, filename:String):Void
    {
    	#if android 
    	if (funcSync == null)
    	{
    			funcSync = JNI.createStaticMethod("com/stencyl/android/TTSAndroid","synthToFile","(Ljava/lang/String;I;Ljava/lang/String;Ljava/lang/String;)V",true);
    	}
    	var args = new Array<Dynamic>();
		args.push(convertToCharCode(text));
		args.push(location);
		args.push(folder);
		args.push(filename);
    	funcSync(args);
    	#end
    }

    	public static function adaptSSMLToEngine(engineName:String)
	{
		switch(engineName)
		{
			case "com.google.android.tts":supportsSSMLTags = false; supportsPhonemeTag = false; tTSEngineType = "crap";
			case "com.svox.pico" : supportsSSMLTags = true; supportsPhonemeTag = true; tTSEngineType = "pico";
			case "edu.cmu.cs.speech.tts.flite" : supportsSSMLTags = false; supportsPhonemeTag = false; tTSEngineType = "crap";
			case "com.redzoc.ramees.tts.espeak" : supportsSSMLTags = true; supportsPhonemeTag = false; tTSEngineType = "espeak";
			default : supportsSSMLTags = false; supportsPhonemeTag = false; tTSEngineType = "crap";
		}

	}





//--------------------------------------  Functions called back by Java --------------------------------------------------------------//

	private static var isTTSSpeakingResult:Bool = false;
	private static var langIdsList:Array<Dynamic>;
	private static var langDisplayNames:Array<Dynamic>;
	private static var installedEnginesResult:Array<Dynamic>;
	

	/**
	* 	Used for Android callbacks from Java
	*/
	public function new()
	{
	}

	/**
	*	function called by JAVA : is TTS speaking
	*/
/*
	public function isTTSSpeaking(result:Bool)
	{
		isTTSSpeakingResult = result;
	
*/

	/**
	*	function called by Java : languages available
	*/

	public function langAvailable(id:String, name:String)
	{
		langIdsList = id.split(";");
		langDisplayNames = name.split(";");
	}

	/**
	*	function called by Java : installed engines
	*/

	public function installedEngines(result:Array<String>)
	{
		//installedEnginesResult = id.split(";");
		trace(result);
	}

	/**
	*	function called by Java : event shouter when TTS finished processing
	*/

	public function finishedShouter(text:String)
	{
		//trace("finished saying "+text);
		setGameAttribute("TTSLastSaidId", text);
		TTS.tTSLastSaidId = text;
		shoutToScene("_customEvent_" + "TTSHasFinishedPlaying");
	}

	/**
	 *  function called by Java : even shouter announcing TTS is loaded
	 */
	
	public function TTSLoadedShouter()
	{
		shoutToScene("_customEvent_" + "TTSLoaded");
	}
 }